"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertHttpResponseToCheckResult = exports.executeHttpRequest = void 0;
const logger_1 = require("../../../logs/logger");
const version_1 = __importDefault(require("../../../common/utils/version"));
const exponential_backoff_1 = require("../../retry/exponential-backoff");
const request_1 = require("../../../common/http/request");
async function executeHttpRequest(checkOptions, httpOptions) {
    logger_1.log.debug({ checkId: checkOptions.id }, 'executing http check');
    try {
        const request = {
            url: httpOptions.url,
            method: httpOptions.method,
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
                'User-Agent': `broker client/${version_1.default} (http check service)`,
            },
        };
        const response = await (0, exponential_backoff_1.retry)(() => (0, request_1.makeSingleRawRequestToDownstream)(request), { retries: 3, operation: `http check ${checkOptions.id}` });
        logger_1.log.trace({ response: response }, 'http check raw response data');
        const checkResult = convertHttpResponseToCheckResult({
            id: checkOptions.id,
            name: checkOptions.name,
            url: httpOptions.url,
            method: httpOptions.method,
        }, response);
        logger_1.log.debug({ checkId: checkOptions.id, status: checkResult.status }, 'completed http check execution');
        return Promise.resolve(checkResult);
    }
    catch (error) {
        const errorMessage = `Error executing check with checkId ${checkOptions.id}`;
        logger_1.log.debug({ error }, errorMessage);
        throw new Error(errorMessage);
    }
}
exports.executeHttpRequest = executeHttpRequest;
function convertHttpResponseToCheckResult(check, response) {
    let status;
    if (response.statusCode &&
        response.statusCode >= 200 &&
        response.statusCode <= 299) {
        status = 'passing';
    }
    else if (response.statusCode == 429) {
        status = 'warning';
    }
    else {
        status = 'error';
    }
    const output = `HTTP ${check.method} ${check.url}: ${response.statusCode} ${response.statusText}, response: ${truncate(JSON.stringify(response.body))}`;
    return {
        id: check.id,
        name: check.name,
        status,
        output,
    };
}
exports.convertHttpResponseToCheckResult = convertHttpResponseToCheckResult;
const truncate = (text) => {
    if (text.length <= 200) {
        return text;
    }
    const truncatedText = text.substring(0, 200);
    return `${truncatedText}... (truncated)`;
};
//# sourceMappingURL=http-executor.js.map