"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executePreflightChecks = exports.preflightChecksEnabled = void 0;
const logger_1 = require("../../logs/logger");
const config_1 = require("./config");
const http_1 = require("./http");
const utils_1 = require("./utils");
function preflightChecksEnabled(config) {
    // preflight checks are enabled per default
    let preflightChecksEnabled = true;
    const preflightChecksEnabledValue = config
        .PREFLIGHT_CHECKS_ENABLED;
    if (typeof preflightChecksEnabledValue !== 'undefined') {
        preflightChecksEnabled =
            preflightChecksEnabledValue.toLowerCase() === 'true' ||
                preflightChecksEnabledValue.toLowerCase() === 'yes';
    }
    logger_1.log.info({ enabled: preflightChecksEnabled }, 'verifying if preflight checks are enabled');
    return preflightChecksEnabled;
}
exports.preflightChecksEnabled = preflightChecksEnabled;
async function executePreflightChecks(config) {
    const preflightChecks = [];
    const httpChecks = (0, http_1.getHttpChecks)(config).filter((c) => c.enabled);
    preflightChecks.push(...httpChecks);
    const configChecks = (0, config_1.getConfigChecks)(config).filter((c) => c.enabled);
    preflightChecks.push(...configChecks);
    const preflightCheckResults = [];
    const results = await Promise.allSettled(preflightChecks.map(async (c) => c.check(config)));
    results.forEach((r) => {
        if (r.status === 'fulfilled') {
            preflightCheckResults.push(r.value);
        }
        else {
            logger_1.log.error({ error: r.reason }, 'Unexpected error when executing checks');
        }
    });
    logPreflightCheckResults(preflightCheckResults);
    return Promise.resolve(preflightCheckResults);
}
exports.executePreflightChecks = executePreflightChecks;
const logPreflightCheckResults = (results) => {
    console.log('\n\n');
    console.log('##############################################################');
    console.log('### PREFLIGHT CHECKS RESULTS - NON BLOCKING');
    console.log('###');
    console.log('### Preflight checks help to catch errors early, upon broker');
    console.log('### client startup. Note, that broker client will start');
    console.log('### whether the checks were successful or not.');
    console.log('###');
    console.log('##############################################################');
    console.log('\n');
    const nonPassingChecksDetails = [
        `#############################################################################################
### PREFLIGHT CHECKS DETAILS
###
### ${(0, utils_1.splitStringIntoLines)(`Get help at https://snyk.io/broker-checks`, 88, '### ')}
###`,
    ];
    const checks = results.map((check) => {
        if (check.status != 'passing') {
            nonPassingChecksDetails.push(`
### [${check.name}] ${check.status}.
### Preflight Check output:
### ${(0, utils_1.splitStringIntoLines)(check.output, 88, '### ')}
### Help: https://snyk.io/broker-checks?${check.id}
###`);
        }
        return { id: check.id, status: check.status };
    });
    nonPassingChecksDetails.push('##########################################################################################');
    console.log('### Preflight checks summary');
    console.table(checks);
    console.log('\n');
    if (checks.some((x) => x.status != 'passing')) {
        console.log(nonPassingChecksDetails.join('\n###'));
    }
    else {
        console.log('#############################################################################################');
        console.log('### All Preflight checks passing');
        console.log('#############################################################################################');
    }
    console.log('\n');
};
//# sourceMappingURL=index.js.map