"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getServerId = exports.highAvailabilityModeEnabled = void 0;
const logger_1 = require("../../logs/logger");
const token_1 = require("../../common/utils/token");
const api_1 = require("./client/api");
const dispatcher_service_1 = require("./dispatcher-service");
function highAvailabilityModeEnabled(config) {
    // high availability mode is disabled per default
    let highAvailabilityModeEnabled = false;
    const highAvailabilityModeEnabledValue = config
        .BROKER_HA_MODE_ENABLED;
    if (typeof highAvailabilityModeEnabledValue !== 'undefined') {
        highAvailabilityModeEnabled =
            highAvailabilityModeEnabledValue.toLowerCase() === 'true' ||
                highAvailabilityModeEnabledValue.toLowerCase() === 'yes';
    }
    logger_1.log.info({ enabled: highAvailabilityModeEnabled }, 'checking for HA mode');
    return highAvailabilityModeEnabled;
}
exports.highAvailabilityModeEnabled = highAvailabilityModeEnabled;
async function getServerId(config, brokerClientId) {
    if (!config.BROKER_TOKEN) {
        logger_1.log.error({ token: config.BROKER_TOKEN }, 'missing client token');
        const error = new Error('BROKER_TOKEN is required to successfully identify itself to the server');
        error.name = 'MISSING_BROKER_TOKEN';
        throw error;
    }
    const defaultBrokerDispatcherBaseUrl = config.API_BASE_URL || config.BROKER_SERVER_URL
        ? config.BROKER_SERVER_URL.replace('//broker.', '//api.')
        : 'https://api.snyk.io';
    const haConfig = getHAConfig(config);
    const baseUrl = haConfig.BROKER_DISPATCHER_BASE_URL || defaultBrokerDispatcherBaseUrl;
    const client = new api_1.HttpDispatcherServiceClient(baseUrl);
    const hashedToken = (0, token_1.hashToken)(config.BROKER_TOKEN);
    const maxRetries = 10;
    for (let attempt = 0; attempt <= maxRetries; attempt++) {
        try {
            return await (0, dispatcher_service_1.getServerIdFromDispatcher)(client, {
                brokerClientId: brokerClientId,
                hashedBrokerToken: hashedToken,
            }, {
                deployment_location: `${config.BROKER_CLIENT_LOCATION || 'snyk-broker-client'}`,
                broker_token_first_char: `${config.BROKER_TOKEN[0]}`,
            });
        }
        catch (err) {
            const timeout = 2 ** attempt * 30000;
            logger_1.log.warn({ attempt, timeout }, `waiting for ${timeout}ms before next Broker Dispatcher API call`);
            Atomics.wait(new Int32Array(new SharedArrayBuffer(4)), 0, 0, timeout);
        }
    }
    return null;
}
exports.getServerId = getServerId;
function getHAConfig(config) {
    return config;
}
//# sourceMappingURL=index.js.map