"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateGitHubTreePayload = exports.signGitHubCommit = exports.commitSigningFilterRules = exports.commitSigningEnabled = exports.gitHubTreeCheckNeeded = exports.gitHubCommitSigningEnabled = void 0;
const logger_1 = require("../../logs/logger");
const commit_1 = require("./github/commit");
const tree_1 = require("./github/tree");
const sign_1 = require("./pgp/sign");
const commit_signing_filter_rules_1 = require("./github/commit-signing-filter-rules");
function gitHubCommitSigningEnabled(config, options) {
    return ((0, exports.commitSigningEnabled)(config) &&
        (0, commit_1.isGitHubCreateCommitEndpoint)(options));
}
exports.gitHubCommitSigningEnabled = gitHubCommitSigningEnabled;
function gitHubTreeCheckNeeded(config, options) {
    return ((0, exports.commitSigningEnabled)(config) &&
        (0, tree_1.isGitHubCreateTreeEndpoint)(options));
}
exports.gitHubTreeCheckNeeded = gitHubTreeCheckNeeded;
const commitSigningEnabled = (config) => {
    let gpgPassphraseConfigured = false;
    if (typeof config.GPG_PASSPHRASE !== 'undefined' &&
        config.GPG_PASSPHRASE !== '') {
        gpgPassphraseConfigured = true;
    }
    let gpgPrivateKeyConfigured = false;
    if (typeof config.GPG_PRIVATE_KEY !== 'undefined' &&
        config.GPG_PRIVATE_KEY !== '') {
        gpgPrivateKeyConfigured = true;
    }
    let gitCommitterNameConfigured = false;
    if (typeof config.GIT_COMMITTER_NAME !== 'undefined' &&
        config.GIT_COMMITTER_NAME !== '') {
        gitCommitterNameConfigured = true;
    }
    let gitCommitterEmailConfigured = false;
    if (typeof config.GIT_COMMITTER_EMAIL !== 'undefined' &&
        config.GIT_COMMITTER_EMAIL !== '') {
        gitCommitterEmailConfigured = true;
    }
    return (gpgPassphraseConfigured &&
        gpgPrivateKeyConfigured &&
        gitCommitterNameConfigured &&
        gitCommitterEmailConfigured);
};
exports.commitSigningEnabled = commitSigningEnabled;
function commitSigningFilterRules() {
    return (0, commit_signing_filter_rules_1.getCommitSigningGitHubFilterRules)();
}
exports.commitSigningFilterRules = commitSigningFilterRules;
/**
 * Sign GitHub commit with PGP private key and set `signature` property
 * for the GitHub commit payload.
 */
async function signGitHubCommit(config, body) {
    const bodyAsString = convertBodyToStringIfNeeded(body);
    const commit = (0, commit_1.convertBodyToGitHubCommitPayload)(bodyAsString, {
        committerName: config.GIT_COMMITTER_NAME,
        committerEmail: config.GIT_COMMITTER_EMAIL,
    });
    logger_1.log.debug({ commit }, 'github commit payload');
    const messageRaw = (0, commit_1.stringifyGitHubCommitPayload)(commit);
    logger_1.log.debug({ messageRaw }, 'raw message before creating pgp signature');
    const armoredKey = (0, sign_1.normalizeArmoredKeyIfNeeded)(config.GPG_PRIVATE_KEY);
    const signature = await (0, sign_1.createSignature)({
        messageRaw: messageRaw,
        privateKey: {
            armoredKey,
            passphrase: config.GPG_PASSPHRASE,
        },
    });
    logger_1.log.debug({ signature }, 'commit pgp signature');
    commit.signature = signature;
    return Promise.resolve(JSON.stringify(commit));
}
exports.signGitHubCommit = signGitHubCommit;
/**
 * Validates GitHub tree object and throw an error if the payload contains symlinks.
 */
function validateGitHubTreePayload(body) {
    const bodyAsString = convertBodyToStringIfNeeded(body);
    const tree = (0, tree_1.convertBodyToGitHubTreePayload)(bodyAsString);
    logger_1.log.debug({ tree }, 'github tree payload');
    (0, tree_1.validateForSymlinksInCreateTree)(tree);
}
exports.validateGitHubTreePayload = validateGitHubTreePayload;
const convertBodyToStringIfNeeded = (body) => {
    if (isUint8Array(body)) {
        return Buffer.from(body).toString();
    }
    else if (typeof body === 'string' || body instanceof String) {
        return body;
    }
    else {
        throw new Error('body must be string or Uint8Array');
    }
};
const isUint8Array = (data) => {
    return !!(data && data instanceof Uint8Array);
};
//# sourceMappingURL=index.js.map