"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeArmoredKeyIfNeeded = exports.validatePrivateKey = exports.createSignature = void 0;
const openpgp_1 = require("openpgp");
const errors_1 = require("./errors");
const pgpPrivateKeyBeginBlock = '-----BEGIN PGP PRIVATE KEY BLOCK-----\n';
const pgpPrivateKeyEndBlock = '-----END PGP PRIVATE KEY BLOCK-----';
async function createSignature(options) {
    validatePrivateKey(options.privateKey);
    const privateKey = await (0, openpgp_1.decryptKey)({
        privateKey: await (0, openpgp_1.readPrivateKey)({
            armoredKey: options.privateKey.armoredKey,
        }),
        passphrase: options.privateKey.passphrase,
    });
    const message = await (0, openpgp_1.createMessage)({ text: options.messageRaw });
    const detachedSignature = await (0, openpgp_1.sign)({
        message,
        signingKeys: privateKey,
        detached: true,
    });
    return Promise.resolve(detachedSignature.replace(/\r\n/g, '\n').trim());
}
exports.createSignature = createSignature;
function validatePrivateKey(pgpPrivateKey) {
    const armoredKey = pgpPrivateKey.armoredKey.trim();
    if (!armoredKey.startsWith(pgpPrivateKeyBeginBlock)) {
        throw new errors_1.PgpPrivateKeyValidationError('missing BEGIN PGP PRIVATE KEY BLOCK');
    }
    if (!armoredKey.endsWith(pgpPrivateKeyEndBlock)) {
        throw new errors_1.PgpPrivateKeyValidationError('missing END PGP PRIVATE KEY BLOCK');
    }
}
exports.validatePrivateKey = validatePrivateKey;
function normalizeArmoredKeyIfNeeded(armoredKey) {
    // passing env vars for Docker containers appends additional backslash
    // for already escaped values (e.g. will be \n -> \\n), this leads to
    // "misformed armor key" error from openpgp library
    return armoredKey.replace(/\\n/g, '\n');
}
exports.normalizeArmoredKeyIfNeeded = normalizeArmoredKeyIfNeeded;
//# sourceMappingURL=sign.js.map