"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const fs_1 = __importDefault(require("fs"));
const logger_1 = require("../../logs/logger");
const camelcase_1 = __importDefault(require("camelcase"));
const SUPPORTED_IAC_EXTENSIONS = ['tf', 'yaml', 'yml', 'tpl', 'json'];
const IAC_SCM_ORIGINS = [
    'GITLAB',
    'AZURE_REPOS_HOST',
    'GITHUB_API',
    'BITBUCKET_API',
];
const CODE_SCM_ORIGINS = ['GITLAB', 'AZURE_REPOS_HOST', 'GITHUB', 'BITBUCKET'];
const SNIPPETS_CODE_SCM_ORIGINS = [
    'GITLAB',
    'AZURE_REPOS_HOST',
    'GITHUB_API',
    'BITBUCKET_API',
];
function nestedCopy(array) {
    return JSON.parse(JSON.stringify(array));
}
function injectRulesAtRuntime(filters, config) {
    var _a;
    if (process.env.ACCEPT_IAC) {
        logger_1.log.info({ accept: process.env.ACCEPT_IAC }, 'Injecting Accept rules for IAC extensions - Possible values tf, yaml, yml, json, tpl');
        const extensions = process.env.ACCEPT_IAC.replace(/\s/g, '')
            .split(',')
            .filter((extension) => SUPPORTED_IAC_EXTENSIONS.includes(extension));
        if (extensions.length <= 0) {
            logger_1.log.error({ accept: extensions }, '[MISCONFIGURATION] None of the requested ACCEPT IAC file extensions is compatible');
        }
        else if (!((_a = filters.private[0].origin) === null || _a === void 0 ? void 0 : _a.includes('AZURE'))) {
            // API endpoints for IAC (github, ghe, bitbucket server), doesn't matter for azure, gitlab
            // file pattern is different for Azure repos, requirements work for all others
            const template = nestedCopy(filters.private.filter((entry) => {
                var _a;
                return entry.method === 'GET' &&
                    ((_a = entry.path) === null || _a === void 0 ? void 0 : _a.includes('requirements')) &&
                    IAC_SCM_ORIGINS.filter((origin) => { var _a; return (_a = entry.origin) === null || _a === void 0 ? void 0 : _a.includes(`{${origin}}`); }).length > 0;
            }));
            for (let i = 0; i < template.length; i++) {
                template[i].path = template[i].path
                    .replace('/requirements', '')
                    .replace('%2Frequirements', '');
                template[i]['//'] = template[i]['//'].replace('determine the full dependency tree', 'scan IAC files');
            }
            for (let i = 0; i < extensions.length; i++) {
                const extensionTemplate = nestedCopy(template);
                extensionTemplate[0].path = extensionTemplate[0].path.replace('txt', extensions[i]);
                extensionTemplate[1].path = extensionTemplate[1].path.replace('txt', extensions[i]);
                filters.private.push(...extensionTemplate);
            }
        }
        else if (filters.private[0].origin.includes('AZURE')) {
            // Copying and modifying in place in array, not doing NestedCopy here
            const templateToModify = filters.private.filter((entry) => {
                var _a;
                return entry.method === 'GET' &&
                    entry.valid &&
                    ((_a = entry.valid[0].values) === null || _a === void 0 ? void 0 : _a.includes('**/requirements/*.txt'));
            });
            for (let i = 0; i < templateToModify.length; i++) {
                for (let j = 0; j < extensions.length; j++) {
                    if (templateToModify[i].valid &&
                        templateToModify[i].valid.length > 0)
                        templateToModify[i].valid[0].values.push(`**/*.${extensions[j]}`);
                    templateToModify[i].valid[0].values.push(`**%2F*.${extensions[j]}`);
                }
            }
        }
        else {
            logger_1.log.error({ accept: process.env.ACCEPT_IAC }, 'Error Unexpected error at rules injection time. Remove ACCEPT_IAC env var to skip injection logic.');
        }
    }
    if (process.env.ACCEPT_LARGE_MANIFESTS) {
        const scmType = CODE_SCM_ORIGINS.find((element) => {
            var _a;
            if ((_a = filters.private[0].origin) === null || _a === void 0 ? void 0 : _a.includes(element)) {
                return true;
            }
        });
        if (scmType === 'GITHUB') {
            logger_1.log.info({ accept: process.env.ACCEPT_LARGE_MANIFESTS }, 'Injecting Accept rules for Large Manifest files');
            const largeManifestRule = {
                '//': 'used to get given manifest file',
                method: 'GET',
                path: '/repos/:owner/:repo/git/blobs/:sha',
                origin: 'https://${GITHUB_TOKEN}@${GITHUB_API}',
            };
            filters.private.push(...[largeManifestRule]);
        }
        else {
            logger_1.log.error({ accept: process.env.ACCEPT_LARGE_MANIFESTS }, 'Large Manifest files Rules is only applicable to Github systems');
        }
    }
    if (process.env.ACCEPT_CODE || process.env.ACCEPT_GIT) {
        logger_1.log.info({ accept: process.env.ACCEPT_CODE || process.env.ACCEPT_GIT }, 'Injecting Accept rules for Code/Git');
        const templateGET = nestedCopy(filters.private.filter((entry) => entry.method === 'GET' &&
            CODE_SCM_ORIGINS.filter((origin) => { var _a; return (_a = entry.origin) === null || _a === void 0 ? void 0 : _a.includes(`{${origin}}`); }).length > 0)[0]);
        const scmType = CODE_SCM_ORIGINS.find((element) => {
            if (templateGET.origin.includes(element)) {
                return true;
            }
        });
        templateGET['//'] = 'allow info refs (for git clone)';
        templateGET.origin = templateGET.origin
            .replace('https://${GITHUB_TOKEN}', 'https://pat:${GITHUB_TOKEN}')
            .replace('https://${GITLAB}', 'https://oauth2:${GITLAB_TOKEN}@${GITLAB}');
        const templatePOST = nestedCopy(templateGET);
        templatePOST.method = 'POST';
        templatePOST['//'] = 'allow git-upload-pack (for git clone)';
        // Code snippets rules
        const templateGETForSnippets = nestedCopy(filters.private.filter((entry) => entry.method === 'GET' &&
            SNIPPETS_CODE_SCM_ORIGINS.filter((origin) => { var _a; return (_a = entry.origin) === null || _a === void 0 ? void 0 : _a.includes(`{${origin}}`); }).length > 0)[0]);
        templateGETForSnippets['//'] = 'needed to load code snippets';
        switch (scmType) {
            case 'AZURE_REPOS_HOST':
                templateGET.path = '*/info/refs*';
                templateGETForSnippets.path =
                    '/:owner/_apis/git/repositories/:repo/items';
                templatePOST.path = '*/git-upload-pack';
                break;
            case 'GITHUB':
                templateGET.path = '*/info/refs*';
                templateGETForSnippets.path = '/repos/:name/:repo/contents/:path';
                templatePOST.path = '*/git-upload-pack';
                break;
            case 'GITLAB':
                templateGET.path = '*/info/refs*';
                templateGETForSnippets.path =
                    '/api/v4/projects/:project/repository/files/:path';
                templatePOST.path = '*/git-upload-pack';
                break;
            case 'BITBUCKET':
                templateGET.path = '*/info/refs*';
                templateGETForSnippets.path =
                    '/projects/:project/repos/:repo/browse*/:file';
                templatePOST.path = '*/git-upload-pack';
                break;
            default:
                logger_1.log.error({}, 'Error writing Code specific rules - Cannot determine SCM type');
        }
        filters.private.push(...[templateGET, templatePOST, templateGETForSnippets]);
    }
    if (process.env.ACCEPT_APPRISK) {
        logger_1.log.info({ accept: process.env.ACCEPT_APPRISK }, 'Injecting Accept rules for AppRisk');
        const type = config.supportedBrokerTypes.find((type) => config[(0, camelcase_1.default)(`BROKER_DOWNSTREAM_TYPE_${type.toLocaleUpperCase()}`)] == 'true');
        if (type) {
            const appRiskRules = require(path_1.default.join(process.cwd(), `defaultFilters/apprisk/${type}.json`));
            // rm entry from filters.private if matching uri in appRiskRules which takes precedence
            const appRiskRulesPathPattern = appRiskRules.map((x) => x.path);
            filters.private = filters.private.filter((x) => !appRiskRulesPathPattern.includes(x.path));
            filters.private.push(...appRiskRules);
        }
    }
    return filters;
}
exports.default = (config, folderLocation = '') => {
    let filters;
    const acceptFilename = config.accept || '';
    if (acceptFilename) {
        const acceptLocation = path_1.default.resolve(folderLocation ? folderLocation : process.cwd(), acceptFilename);
        filters = js_yaml_1.default.safeLoad(fs_1.default.readFileSync(acceptLocation, 'utf8'));
    }
    // If user brings an accept json, skip the IAC|CODE rules injection logic
    // If going through the effort of loading a separate file, add your rules there
    if (process.env.ACCEPT === 'accept.json') {
        filters = injectRulesAtRuntime(filters, config);
    }
    else {
        logger_1.log.info({ accept: process.env.ACCEPT }, 'Custom accept json, skipping filter rule injection');
    }
    return filters;
};
//# sourceMappingURL=filter-rules-loading.js.map