"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeSingleRawRequestToDownstream = exports.makeStreamingRequestToDownstream = exports.makeRequestToDownstream = void 0;
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const proxy_from_env_1 = require("proxy-from-env");
const global_agent_1 = require("global-agent");
const logger_1 = require("../../logs/logger");
const config_1 = require("../config");
const utils_1 = require("./utils");
const MAX_RETRY = config_1.config.MAX_RETRY || 3;
if (process.env.HTTP_PROXY || process.env.http_proxy) {
    process.env.HTTP_PROXY = process.env.HTTP_PROXY || process.env.http_proxy;
}
if (process.env.HTTPS_PROXY || process.env.https_proxy) {
    process.env.HTTPS_PROXY = process.env.HTTPS_PROXY || process.env.https_proxy;
}
if (process.env.NP_PROXY || process.env.no_proxy) {
    process.env.NO_PROXY = process.env.NO_PROXY || process.env.no_proxy;
}
const makeRequestToDownstream = async (req, retries = MAX_RETRY) => {
    const localRequest = req;
    if (config_1.config.INSECURE_DOWNSTREAM) {
        localRequest.url = (0, utils_1.switchToInsecure)(localRequest.url);
    }
    const proxyUri = (0, proxy_from_env_1.getProxyForUrl)(localRequest.url);
    if (proxyUri) {
        (0, global_agent_1.bootstrap)({
            environmentVariableNamespace: '',
        });
    }
    const httpClient = localRequest.url.startsWith('https') ? https_1.default : http_1.default;
    const options = {
        method: localRequest.method,
        headers: localRequest.headers,
    };
    return new Promise((resolve, reject) => {
        try {
            const request = httpClient.request(localRequest.url, options, (response) => {
                let data = '';
                // A chunk of data has been received.
                response.on('data', (chunk) => {
                    data += chunk;
                });
                // The whole response has been received.
                response.on('end', () => {
                    if (response.statusCode &&
                        response.statusCode >= 200 &&
                        response.statusCode < 300) {
                        logger_1.log.trace({ statusCode: response.statusCode, url: localRequest.url }, `Successful request`);
                    }
                    else {
                        logger_1.log.debug({ statusCode: response.statusCode, url: localRequest.url }, `Non 2xx HTTP Code Received`);
                    }
                    resolve({
                        headers: response.headers,
                        statusCode: response.statusCode,
                        body: data,
                    });
                });
                response.on('error', (error) => {
                    if (retries > 0) {
                        logger_1.log.warn({ msg: localRequest.url }, `Downstream Response failed. Retrying after 500ms...`);
                        setTimeout(() => {
                            resolve((0, exports.makeRequestToDownstream)(localRequest, retries - 1));
                        }, 500); // Wait for 0.5 second before retrying
                    }
                    else {
                        logger_1.log.error({ error }, `Error getting response from downstream. Giving up after ${MAX_RETRY} retries.`);
                        reject(error);
                    }
                });
            });
            // An error occurred while fetching.
            request.on('error', (error) => {
                if (retries > 0) {
                    logger_1.log.warn({ url: localRequest.url, err: error }, `Request failed. Retrying after 500ms...`);
                    setTimeout(() => {
                        resolve((0, exports.makeRequestToDownstream)(localRequest, retries - 1));
                    }, 500); // Wait for 0.5 second before retrying
                }
                else {
                    logger_1.log.error({ url: localRequest.url, err: error }, `Error making streaming request to downstream. Giving up after ${MAX_RETRY} retries.`);
                    reject(error);
                }
            });
            if (localRequest.body) {
                request.write(localRequest.body);
            }
            request.end();
        }
        catch (err) {
            reject(err);
        }
    });
};
exports.makeRequestToDownstream = makeRequestToDownstream;
const makeStreamingRequestToDownstream = (req, retries = MAX_RETRY) => {
    const localRequest = req;
    if (config_1.config.INSECURE_DOWNSTREAM) {
        localRequest.url = (0, utils_1.switchToInsecure)(localRequest.url);
    }
    const proxyUri = (0, proxy_from_env_1.getProxyForUrl)(localRequest.url);
    if (proxyUri) {
        (0, global_agent_1.bootstrap)({
            environmentVariableNamespace: '',
        });
    }
    const httpClient = localRequest.url.startsWith('https') ? https_1.default : http_1.default;
    const options = {
        method: localRequest.method,
        headers: localRequest.headers,
    };
    return new Promise((resolve, reject) => {
        try {
            const request = httpClient.request(localRequest.url, options, (response) => {
                if (response.statusCode &&
                    response.statusCode >= 200 &&
                    response.statusCode < 300) {
                    logger_1.log.info({
                        statusCode: response.statusCode,
                        url: localRequest.url,
                        headers: config_1.config.LOG_INFO_VERBOSE ? response.headers : {},
                    }, `Successful downstream request`);
                }
                else {
                    logger_1.log.warn({
                        statusCode: response.statusCode,
                        url: localRequest.url,
                        headers: response.headers,
                    }, `Non 2xx HTTP Code Received`);
                }
                response.on('error', (error) => {
                    if (retries > 0) {
                        logger_1.log.warn({ msg: localRequest.url }, `Downstream Response failed. Retrying after 500ms...`);
                        setTimeout(() => {
                            resolve((0, exports.makeStreamingRequestToDownstream)(localRequest, retries - 1));
                        }, 500); // Wait for 0.5 second before retrying
                    }
                    else {
                        logger_1.log.error({ error }, `Error getting response from downstream. Giving up after ${MAX_RETRY} retries.`);
                        reject(error);
                    }
                });
                resolve(response);
            });
            request.on('error', (error) => {
                if (retries > 0) {
                    logger_1.log.warn({ url: req.url, err: error }, `Request failed. Retrying after 500ms...`);
                    setTimeout(() => {
                        resolve((0, exports.makeStreamingRequestToDownstream)(localRequest, retries - 1));
                    }, 500); // Wait for 0.5 second before retrying
                }
                else {
                    logger_1.log.error({ url: localRequest.url, err: error }, `Error making request to downstream. Giving up after ${MAX_RETRY} retries.`);
                    reject(error);
                }
            });
            if (localRequest.body) {
                request.write(localRequest.body);
            }
            request.end();
        }
        catch (err) {
            reject(err);
        }
    });
};
exports.makeStreamingRequestToDownstream = makeStreamingRequestToDownstream;
const makeSingleRawRequestToDownstream = async (req) => {
    const localRequest = req;
    if (config_1.config.INSECURE_DOWNSTREAM) {
        localRequest.url = (0, utils_1.switchToInsecure)(localRequest.url);
    }
    const proxyUri = (0, proxy_from_env_1.getProxyForUrl)(localRequest.url);
    if (proxyUri) {
        (0, global_agent_1.bootstrap)({
            environmentVariableNamespace: '',
        });
    }
    const httpClient = localRequest.url.startsWith('https') ? https_1.default : http_1.default;
    const options = {
        method: localRequest.method,
        headers: localRequest.headers,
    };
    return new Promise((resolve, reject) => {
        try {
            const request = httpClient.request(localRequest.url, options, (response) => {
                let data = '';
                // A chunk of data has been received.
                response.on('data', (chunk) => {
                    data += chunk;
                });
                // The whole response has been received.
                response.on('end', () => {
                    resolve({
                        headers: response.headers,
                        statusCode: response.statusCode,
                        statusText: response.statusMessage || '',
                        body: data,
                    });
                });
                // An error occurred while fetching.
                response.on('error', (error) => {
                    logger_1.log.error({ error }, 'Error making request to downstream.');
                    reject(error);
                });
            });
            request.on('error', (error) => {
                logger_1.log.error({ error }, 'Error making request to downstream.');
                reject(error);
            });
            if (localRequest.body) {
                request.write(localRequest.body);
            }
            request.end();
        }
        catch (err) {
            reject(err);
        }
    });
};
exports.makeSingleRawRequestToDownstream = makeSingleRawRequestToDownstream;
//# sourceMappingURL=request.js.map