"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardHttpRequest = void 0;
const filtersAsync_1 = require("../filter/filtersAsync");
const undefsafe_1 = __importDefault(require("undefsafe"));
const uuid_1 = require("uuid");
const logger_1 = require("../../logs/logger");
const stream_1 = __importDefault(require("stream"));
const metrics_1 = require("../utils/metrics");
const server_post_stream_handler_1 = require("../http/server-post-stream-handler");
// 1. Request coming in over HTTP conn (logged)
// 2. Filter for rule match (log and block if no match)
// 3. Relay over websocket conn (logged)
// 4. Get response over websocket conn (logged)
// 5. Send response over HTTP conn
const forwardHttpRequest = (filterRules) => {
    const filters = (0, filtersAsync_1.loadFilters)(filterRules);
    return (req, res) => {
        var _a, _b;
        var _c;
        // If this is the server, we should receive a Snyk-Request-Id header from upstream
        // If this is the client, we will have to generate one
        (_c = req.headers)['snyk-request-id'] || (_c['snyk-request-id'] = (0, uuid_1.v4)());
        const logContext = {
            url: req.url,
            requestMethod: req.method,
            requestHeaders: req.headers,
            requestId: req.headers['snyk-request-id'] &&
                Array.isArray(req.headers['snyk-request-id'])
                ? req.headers['snyk-request-id'].join(',')
                : req.headers['snyk-request-id'] || '',
            maskedToken: req['maskedToken'],
            hashedToken: req['hashedToken'],
        };
        const simplifiedContext = logContext;
        delete simplifiedContext.requestHeaders;
        logger_1.log.info(simplifiedContext, '[HTTP Flow] Received request');
        const filterResponse = filters(req);
        const makeWebsocketRequestWithStreamingResponse = (result) => {
            req.url = result.url;
            logContext.ioUrl = result.url;
            const streamingID = (0, uuid_1.v4)();
            const streamBuffer = new stream_1.default.PassThrough({ highWaterMark: 1048576 });
            streamBuffer.on('error', (error) => {
                // This may be a duplicate error, as the most likely cause of this is the POST handler calling destroy.
                logger_1.log.error({
                    ...logContext,
                    error,
                    stackTrace: new Error('stacktrace generator').stack,
                }, '[HTTP Flow][Relay] Error piping POST response stream through to HTTP response');
                res.destroy(error);
            });
            logContext.streamingID = streamingID;
            logger_1.log.debug(logContext, '[HTTP Flow][Relay] Sending request over websocket connection expecting POST stream response');
            server_post_stream_handler_1.streamsStore.set(streamingID, {
                response: res,
                streamBuffer,
                streamSize: 0,
            });
            streamBuffer.pipe(res);
            res.locals.io.send('request', {
                url: req.url,
                method: req.method,
                body: req.body,
                headers: req.headers,
                streamingID,
            });
            (0, metrics_1.incrementWebSocketRequestsTotal)(false, 'outbound-request');
            return;
        };
        const makeWebsocketRequestWithWebsocketResponse = (result) => {
            req.url = result.url;
            logContext.ioUrl = result.url;
            logger_1.log.debug(logContext, '[HTTP Flow][Relay] Sending request over websocket connection expecting Websocket response');
            // relay the http request over the websocket, handle websocket response
            res.locals.io.send('request', {
                url: req.url,
                method: req.method,
                body: req.body,
                headers: req.headers,
                streamingID: '',
            }, (ioResponse) => {
                logContext.responseStatus = ioResponse.status;
                logContext.responseHeaders = ioResponse.headers;
                logContext.responseBodyType = typeof ioResponse.body;
                const logMsg = '[HTTP Flow][Relay] Return response from Websocket back to HTTP connection';
                if (ioResponse.status <= 299) {
                    logger_1.log.debug(logContext, logMsg);
                    let responseBodyString = '';
                    if (typeof ioResponse.body === 'string') {
                        responseBodyString = ioResponse.body;
                    }
                    else if (typeof ioResponse.body === 'object') {
                        responseBodyString = JSON.stringify(ioResponse.body);
                    }
                    if (responseBodyString) {
                        const responseBodyBytes = Buffer.byteLength(responseBodyString, 'utf-8');
                        (0, metrics_1.observeResponseSize)({
                            bytes: responseBodyBytes,
                            isStreaming: false,
                        });
                    }
                    else {
                        // fallback metric to let us know if we're recording all response sizes
                        // we expect to remove this should it report 0
                        (0, metrics_1.incrementUnableToSizeResponse)();
                    }
                }
                else {
                    const errorLogMsg = '[HTTP Flow][Relay] Non 2xx response from Websocket back to HTTP connection';
                    logContext.ioErrorType = ioResponse.errorType;
                    logContext.ioOriginalBodySize = ioResponse.originalBodySize;
                    logger_1.log.warn(logContext, errorLogMsg);
                }
                const httpResponse = res
                    .status(ioResponse.status)
                    .set(ioResponse.headers);
                const encodingType = (0, undefsafe_1.default)(ioResponse, 'headers.transfer-encoding');
                try {
                    // keep chunked http requests without content-length header
                    if (encodingType === 'chunked') {
                        httpResponse.write(ioResponse.body);
                        httpResponse.end();
                    }
                    else {
                        httpResponse.send(ioResponse.body);
                    }
                }
                catch (err) {
                    logger_1.log.error({
                        ...logContext,
                        encodingType,
                        err,
                        stackTrace: new Error('stacktrace generator').stack,
                    }, '[HTTP Flow][Relay] Error forwarding response from Web Socket to HTTP connection');
                }
            });
            (0, metrics_1.incrementWebSocketRequestsTotal)(false, 'outbound-request');
        };
        if (!filterResponse) {
            (0, metrics_1.incrementHttpRequestsTotal)(true, 'inbound-request');
            const reason = 'Request does not match any accept rule, blocking HTTP request';
            logContext.error = 'blocked';
            logger_1.log.warn(logContext, reason);
            // TODO: respect request headers, block according to content-type
            return res.status(401).send({ message: 'blocked', reason, url: req.url });
        }
        else {
            (0, metrics_1.incrementHttpRequestsTotal)(false, 'inbound-request');
            if (filterResponse.stream ||
                ((_b = (_a = res === null || res === void 0 ? void 0 : res.locals) === null || _a === void 0 ? void 0 : _a.capabilities) === null || _b === void 0 ? void 0 : _b.includes('post-streams'))) {
                makeWebsocketRequestWithStreamingResponse(filterResponse);
            }
            else {
                makeWebsocketRequestWithWebsocketResponse(filterResponse);
            }
        }
    };
};
exports.forwardHttpRequest = forwardHttpRequest;
//# sourceMappingURL=forwardHttpRequest.js.map