"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardWebSocketRequest = void 0;
const token_1 = require("../utils/token");
const logger_1 = require("../../logs/logger");
const downstream_post_stream_to_server_1 = require("../http/downstream-post-stream-to-server");
const metrics_1 = require("../utils/metrics");
const filtersAsync_1 = require("../filter/filtersAsync");
const prepareRequest_1 = require("./prepareRequest");
const requestsHelper_1 = require("./requestsHelper");
const forwardWebSocketRequest = (options, io) => {
    // 1. Request coming in over websocket conn (logged)
    // 2. Filter for rule match (log and block if no match)
    // 3. Relay over HTTP conn (logged)
    // 4. Get response over HTTP conn (logged)
    // 5. Send response over websocket conn
    var _a;
    //   const filters = Filters(options.filters?.private);
    const filters = (0, filtersAsync_1.loadFilters)((_a = options.filters) === null || _a === void 0 ? void 0 : _a.private);
    return (brokerToken) => async (payload, emit) => {
        var _a, _b, _c, _d;
        const requestId = payload.headers['snyk-request-id'];
        const logContext = {
            url: payload.url,
            requestMethod: payload.method,
            requestHeaders: payload.headers,
            requestId,
            streamingID: payload.streamingID,
            maskedToken: (0, token_1.maskToken)(brokerToken),
            hashedToken: (0, token_1.hashToken)(brokerToken),
            transport: (_c = (_b = (_a = io === null || io === void 0 ? void 0 : io.socket) === null || _a === void 0 ? void 0 : _a.transport) === null || _b === void 0 ? void 0 : _b.name) !== null && _c !== void 0 ? _c : 'unknown',
        };
        if (!requestId) {
            // This should be a warning but older clients won't send one
            // TODO make this a warning when significant majority of clients are on latest version
            logger_1.log.debug(logContext, 'Header Snyk-Request-Id not included in headers passed through');
        }
        // It clarifies which emit handler is for websockets vs POST response
        const websocketEmit = emit;
        const postOverrideEmit = (responseData, isResponseFromRequestModule = false) => {
            try {
                logContext.requestMethod = '';
                logContext.requestHeaders = {};
                const postHandler = new downstream_post_stream_to_server_1.BrokerServerPostResponseHandler(logContext, options.config, brokerToken, options.config.serverId, requestId);
                if (isResponseFromRequestModule) {
                    logger_1.log.debug(logContext, '[Websocket Flow] Posting HTTP streaming response back to Broker Server');
                    postHandler.forwardRequest(responseData, payload.streamingID);
                }
                else {
                    logger_1.log.debug(logContext, '[Websocket Flow] Posting HTTP streaming response back to Broker Server');
                    // Only for responses generated internally in the Broker Client/Server
                    postHandler.sendData(responseData, payload.streamingID);
                }
            }
            catch (err) {
                logger_1.log.error({ err }, `Error Posting via Emit callback.`);
            }
        };
        const legacyOverrideEmit = (responseData, isResponseFromRequestModule = false) => {
            // Traffic over websockets
            if (payload.streamingID) {
                if (isResponseFromRequestModule) {
                    (0, requestsHelper_1.legacyStreaming)(logContext, responseData, options.config, io, payload.streamingID);
                }
                else {
                    io.send('chunk', payload.streamingID, '', false, {
                        status: responseData.status,
                        headers: responseData.headers,
                    });
                    io.send('chunk', payload.streamingID, JSON.stringify(responseData.body), true);
                }
            }
            else {
                logger_1.log.debug({ ...logContext, responseData }, '[Websocket Flow] (Legacy) Sending fixed response over WebSocket connection');
                websocketEmit(responseData);
            }
        };
        if ((_d = io === null || io === void 0 ? void 0 : io.capabilities) === null || _d === void 0 ? void 0 : _d.includes('receive-post-streams')) {
            // Traffic over HTTP Post
            emit = postOverrideEmit;
        }
        else {
            emit = legacyOverrideEmit;
        }
        const simplifiedContext = logContext;
        delete simplifiedContext.requestHeaders;
        logger_1.log.info(simplifiedContext, `[Websocket Flow] Received request from ${process.env.BROKER_TYPE == 'client' ? 'server' : 'client'}`);
        const filterResponse = filters(payload);
        if (!filterResponse) {
            (0, metrics_1.incrementWebSocketRequestsTotal)(true, 'inbound-request');
            const reason = '[Websocket Flow][Blocked Request] Does not match any accept rule';
            logContext.error = 'Blocked by filter rules';
            logger_1.log.warn(logContext, reason);
            return emit({
                status: 401,
                body: {
                    message: 'blocked',
                    reason,
                    url: payload.url,
                },
            });
        }
        else {
            (0, metrics_1.incrementWebSocketRequestsTotal)(false, 'inbound-request');
            const preparedRequest = await (0, prepareRequest_1.prepareRequestFromFilterResult)(filterResponse, payload, logContext, options, brokerToken, io === null || io === void 0 ? void 0 : io.socketType);
            (0, metrics_1.incrementHttpRequestsTotal)(false, 'outbound-request');
            payload.streamingID
                ? await (0, requestsHelper_1.makePostStreamingRequest)(preparedRequest.req, emit, logContext)
                : await (0, requestsHelper_1.makeLegacyRequest)(preparedRequest.req, emit, logContext, options);
        }
    };
};
exports.forwardWebSocketRequest = forwardWebSocketRequest;
//# sourceMappingURL=forwardWebsocketRequest.js.map