"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareRequestFromFilterResult = void 0;
const url_1 = require("url");
const version_1 = __importDefault(require("../utils/version"));
const try_json_parse_1 = __importDefault(require("../utils/try-json-parse"));
const replace_vars_1 = require("../utils/replace-vars");
const undefsafe_1 = __importDefault(require("undefsafe"));
const logger_1 = require("../../logs/logger");
const scm_1 = require("../../client/scm");
const prepareRequestFromFilterResult = async (result, payload, logContext, options, brokerToken, socketType) => {
    var _a;
    let errorPreparing = null;
    if (result.url.startsWith('http') === false) {
        result.url = 'https://' + result.url;
        logContext.resultUrlSchemeAdded = true;
    }
    logContext.httpUrl = result.url;
    if (!payload.headers['user-agent']) {
        payload.headers['user-agent'] = 'Snyk Broker ' + version_1.default;
        logContext.userAgentHeaderSet = true;
    }
    if (result.auth) {
        payload.headers['authorization'] = result.auth;
        logContext.authHeaderSetByRuleAuth = true;
    }
    else {
        const parsed = (0, url_1.parse)(result.url);
        if (parsed.auth) {
            // if URL contains basic auth,
            // remove authorization header to prefer auth on the URL.
            if (parsed.auth.includes(':')) {
                delete payload.headers['authorization'];
            }
            // if URL contains token auth,
            // put the token in the authorization header
            // instead of on the URL.
            else {
                payload.headers['authorization'] = `token ${parsed.auth}`;
                // then strip from the url
                parsed.auth = null;
                result.url = (0, url_1.format)(parsed);
            }
            logContext.authHeaderSetByRuleUrl = true;
        }
    }
    // if the request is all good - and at this point it is, we'll check
    // whether we want to do variable substitution on the body
    //
    // Variable substitution - for those who forgot - is substituting a part
    // of a given string (e.g. "${SOME_ENV_VAR}/rest/of/string")
    // with an env var of the same name (SOME_ENV_VAR).
    // This is used (for example) to substitute the snyk url
    // with the broker's url when defining the target for an incoming webhook.
    if (!options.config.disableBodyVarsSubstitution && payload.body) {
        const parsedBody = (0, try_json_parse_1.default)(payload.body);
        if (parsedBody.BROKER_VAR_SUB) {
            logContext.bodyVarsSubstitution = parsedBody.BROKER_VAR_SUB;
            for (const path of parsedBody.BROKER_VAR_SUB) {
                let source = (0, undefsafe_1.default)(parsedBody, path); // get the value
                source = (0, replace_vars_1.replace)(source, options.config); // replace the variables
                (0, undefsafe_1.default)(parsedBody, path, source); // put it back in
            }
            payload.body = JSON.stringify(parsedBody);
        }
    }
    if (!options.config.disableHeaderVarsSubstitution &&
        payload.headers &&
        payload.headers['x-broker-var-sub']) {
        // check whether we want to do variable substitution on the headers
        logContext.headerVarsSubstitution = payload.headers['x-broker-var-sub'];
        for (const path of payload.headers['x-broker-var-sub'].split(',')) {
            let source = (0, undefsafe_1.default)(payload.headers, path.trim()); // get the value
            source = (0, replace_vars_1.replace)(source, options.config); // replace the variables
            (0, undefsafe_1.default)(payload.headers, path.trim(), source); // put it back in
        }
    }
    // remove headers that we don't want to relay
    // (because they corrupt the request)
    [
        'x-forwarded-for',
        'x-forwarded-proto',
        'content-length',
        'host',
        'accept-encoding',
        'content-encoding',
    ].map((_) => delete payload.headers[_]);
    if (options.config.removeXForwardedHeaders === 'true') {
        for (const key in payload.headers) {
            if (key.startsWith('x-forwarded-')) {
                delete payload.headers[key];
            }
        }
        if (payload.headers['forwarded']) {
            delete payload.headers['forwarded'];
        }
    }
    if (brokerToken && socketType === 'server') {
        Object.assign(payload.headers, { 'X-Broker-Token': brokerToken });
    }
    logger_1.log.debug(logContext, '[Relay] Preparing Downstream Request');
    // Sometimes we receive the body as a {type, data} object
    // Unsure why - possibly Primus?
    if (((_a = payload.body) === null || _a === void 0 ? void 0 : _a.type) === 'Buffer')
        payload.body = Buffer.of(payload.body.data);
    // Request library is buggy and will throw an error if we're POST'ing an empty body without an explicit Content-Length header
    if (!payload.body || payload.body.length === 0) {
        payload.headers['Content-Length'] = '0';
    }
    else {
        payload.headers['Content-length'] = payload.body.length;
    }
    payload.headers['connection'] = 'Keep-Alive';
    payload.headers['Keep-Alive'] = 'timeout=60, max=1000';
    if ((0, scm_1.gitHubTreeCheckNeeded)(options.config, {
        method: payload.method,
        url: payload.url,
    })) {
        try {
            (0, scm_1.validateGitHubTreePayload)(payload.body);
        }
        catch (error) {
            logger_1.log.error({ error }, 'error while checking github tree payload for symlinks');
            errorPreparing = {
                status: 401,
                errorMsg: error.message,
            };
        }
    }
    if ((0, scm_1.gitHubCommitSigningEnabled)(options.config, {
        method: payload.method,
        url: payload.url,
    })) {
        try {
            payload.body = await (0, scm_1.signGitHubCommit)(options.config, payload.body);
        }
        catch (error) {
            logger_1.log.error({ error }, 'error while signing github commit');
        }
    }
    const req = {
        url: result.url,
        headers: payload.headers,
        method: payload.method,
        body: payload.body,
    };
    return { req, error: errorPreparing };
};
exports.prepareRequestFromFilterResult = prepareRequestFromFilterResult;
//# sourceMappingURL=prepareRequest.js.map