"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.legacyStreaming = exports.makeLegacyRequest = exports.makePostStreamingRequest = void 0;
const request_1 = require("../http/request");
const logger_1 = require("../../logs/logger");
const log_1 = require("../../logs/log");
const json_1 = require("../utils/json");
const replace_vars_1 = require("../utils/replace-vars");
const makePostStreamingRequest = async (req, emitCallback, logContext) => {
    // this is a streaming request for binary data
    logger_1.log.debug(logContext, '[Downstream] Make Post stream request to downstream');
    try {
        const downstreamRequestIncomingResponse = await (0, request_1.makeStreamingRequestToDownstream)(req);
        emitCallback(downstreamRequestIncomingResponse, true);
    }
    catch (e) {
        logger_1.log.error({
            ...logContext,
            error: e,
            stackTrace: new Error('stacktrace generator').stack,
        }, '[Downstream] Caught error making streaming request to downstream ');
    }
    return;
};
exports.makePostStreamingRequest = makePostStreamingRequest;
const makeLegacyRequest = async (req, emitCallback, logContext, options) => {
    // not main http post flow
    try {
        const response = await (0, request_1.makeRequestToDownstream)(req);
        const contentLength = response.body.length;
        // Note that the other side of the request will also check the length and will also reject it if it's too large
        // Set to 20MB even though the server is 21MB because the server looks at the total data travelling through the websocket,
        // not just the size of the body, so allow 1MB for miscellaneous data (e.g., headers, Primus overhead)
        const maxLength = parseInt(options.config.socketMaxResponseLength) || 20971520;
        if (contentLength && contentLength > maxLength) {
            const errorMessage = `body size of ${contentLength} is greater than max allowed of ${maxLength} bytes`;
            (0, log_1.logError)(logContext, {
                errorMessage,
            });
            return emitCallback({
                status: 502,
                errorType: 'BODY_TOO_LARGE',
                originalBodySize: contentLength,
                body: {
                    message: errorMessage,
                },
            });
        }
        const status = (response && response.statusCode) || 500;
        if (options.config.RES_BODY_URL_SUB && (0, json_1.isJson)(response.headers)) {
            const replaced = (0, replace_vars_1.replaceUrlPartialChunk)(response.body, null, options.config);
            response.body = replaced.newChunk;
        }
        if (status > 404) {
            logger_1.log.warn({
                statusCode: response.statusCode,
                responseHeaders: response.headers,
                url: req.url,
            }, `[Websocket Flow][Inbound] Unexpected status code for relayed request`);
        }
        (0, log_1.logResponse)(logContext, status, response, options.config);
        emitCallback({ status, body: response.body, headers: response.headers });
    }
    catch (error) {
        (0, log_1.logError)(logContext, error);
        return emitCallback({
            status: 500,
            body: error,
        });
    }
};
exports.makeLegacyRequest = makeLegacyRequest;
const legacyStreaming = (logContext, rqst, config, io, streamingID) => {
    let prevPartialChunk;
    let isResponseJson;
    logger_1.log.warn(logContext, 'server did not advertise received-post-streams capability - falling back to legacy streaming');
    // Fall back to older streaming method if somehow connected to older server version
    rqst
        .on('response', (response) => {
        const status = (response && response.statusCode) || 500;
        (0, log_1.logResponse)(logContext, status, response, config);
        isResponseJson = (0, json_1.isJson)(response.headers);
        io.send('chunk', streamingID, '', false, {
            status,
            headers: response.headers,
        });
    })
        .on('data', (chunk) => {
        if (config.RES_BODY_URL_SUB && isResponseJson) {
            const { newChunk, partial } = (0, replace_vars_1.replaceUrlPartialChunk)(Buffer.from(chunk).toString(), prevPartialChunk, config);
            prevPartialChunk = partial;
            chunk = newChunk;
        }
        io.send('chunk', streamingID, chunk, false);
    })
        .on('end', () => {
        io.send('chunk', streamingID, '', true);
    })
        .on('error', (error) => {
        (0, log_1.logError)(logContext, error);
        io.send('chunk', streamingID, error.message, true, {
            status: 500,
        });
    });
};
exports.legacyStreaming = legacyStreaming;
//# sourceMappingURL=requestsHelper.js.map