"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlePostResponse = void 0;
const server_post_stream_handler_1 = require("../../common/http/server-post-stream-handler");
const logger_1 = require("../../logs/logger");
const token_1 = require("../utils/token");
const metrics_1 = require("../../common/utils/metrics");
const handlePostResponse = (req, res) => {
    (0, metrics_1.incrementHttpRequestsTotal)(false, 'data-response');
    const token = req.params.brokerToken;
    const streamingID = req.params.streamingId;
    const desensitizedToken = (0, token_1.getDesensitizedToken)(token);
    const logContext = {
        hashedToken: desensitizedToken.hashedToken,
        maskToken: desensitizedToken.maskedToken,
        streamingID,
        requestId: req.headers['snyk-request-id'],
    };
    logger_1.log.info(logContext, 'Handling response-data request');
    req['maskedToken'] = desensitizedToken.maskedToken;
    req['hashedToken'] = desensitizedToken.hashedToken;
    const streamHandler = server_post_stream_handler_1.StreamResponseHandler.create(streamingID);
    if (!streamHandler) {
        logger_1.log.error(logContext, 'unable to find request matching streaming id');
        res
            .status(500)
            .json({ message: 'unable to find request matching streaming id' });
        return;
    }
    let statusAndHeaders = '';
    let statusAndHeadersSize = -1;
    req
        .on('data', function (data) {
        try {
            logger_1.log.trace({ ...logContext, dataLength: data.length }, 'Received data event');
            let bytesRead = 0;
            if (statusAndHeadersSize === -1) {
                bytesRead += 4;
                statusAndHeadersSize = data.readUInt32LE();
                logger_1.log.debug({ ...logContext, statusAndHeadersSize }, 'request metadata size read from stream');
            }
            if (statusAndHeadersSize > 0 &&
                statusAndHeaders.length < statusAndHeadersSize) {
                const endPosition = Math.min(bytesRead + statusAndHeadersSize - statusAndHeaders.length, data.length);
                logger_1.log.trace({ ...logContext, bytesRead, endPosition }, 'Reading ioJson');
                statusAndHeaders += data.toString('utf8', bytesRead, endPosition);
                bytesRead = endPosition;
                if (statusAndHeaders.length === statusAndHeadersSize) {
                    logger_1.log.trace({ ...logContext, statusAndHeaders }, 'Converting to json');
                    const statusAndHeadersJson = JSON.parse(statusAndHeaders);
                    const logData = {
                        ...logContext,
                        responseStatus: statusAndHeadersJson.status,
                        responseHeaders: statusAndHeadersJson.headers,
                    };
                    const logMessage = 'Handling response-data request - io bits';
                    if (statusAndHeadersJson.status > 299 &&
                        statusAndHeadersJson.status !== 404) {
                        logger_1.log.info(logData, logMessage);
                    }
                    else {
                        logger_1.log.debug(logData, logMessage);
                    }
                    streamHandler.writeStatusAndHeaders(statusAndHeadersJson);
                }
                else {
                    logger_1.log.trace({
                        ...logContext,
                        currentSize: statusAndHeaders.length,
                        expectedSize: statusAndHeadersSize,
                    }, 'Was unable to fit all information into a single data object');
                }
            }
            if (bytesRead < data.length) {
                logger_1.log.trace(logContext, 'Handling response-data request - data part');
                streamHandler.writeChunk(data.subarray(bytesRead, data.length), (streamBuffer) => {
                    logger_1.log.trace(logContext, 'pausing request stream');
                    req.pause();
                    streamBuffer.once('drain', () => {
                        logger_1.log.trace(logContext, 'resuming request stream');
                        req.resume();
                    });
                });
            }
        }
        catch (e) {
            logger_1.log.error({ ...logContext, statusAndHeaders, statusAndHeadersSize, error: e }, 'caught error handling data event for streaming HTTP response');
        }
    })
        .on('end', function () {
        logger_1.log.debug(logContext, 'Handling response-data request - end part');
        streamHandler.finished();
        res.status(200).json({});
    })
        .on('error', (err) => {
        logger_1.log.error({ ...logContext, error: err }, 'received error handling POST from client');
        streamHandler.destroy(err);
        res.status(500).json({ err });
    });
};
exports.handlePostResponse = handlePostResponse;
//# sourceMappingURL=postResponseHandler.js.map